@@ -1,6 +1,7 @@
 /*************************************************************************/
 /* Copyright (c) 2004                                                    */
 /* Daniel Sleator, David Temperley, and John Lafferty                    */
+/* Copyright (c) 2010 Linas Vepstas                                      */
 /* All rights reserved                                                   */
 /*                                                                       */
 /* Use of the link grammar parsing system is subject to the terms of the */
@@ -107,25 +108,35 @@ static int x_hash(int lw, int rw, Connector *le, Connector *re, int cost, Parse_
 }
 
 /**
+ * Allocate the parse info struct
+ *
  * A piecewise exponential function determines the size of the hash
  * table.  Probably should make use of the actual number of disjuncts,
  * rather than just the number of words.
  */
-void init_x_table(Sentence sent)
+Parse_info parse_info_new(int nwords)
 {
-	int i, x_table_size;
+	int x_table_size;
 	Parse_info pi;
 
-	assert(sent->parse_info == NULL, "Parse_info is not NULL");
-
-	pi = sent->parse_info = (Parse_info) xalloc(sizeof(struct Parse_info_struct));
-	pi->N_words = sent->length;
+	pi = (Parse_info) xalloc(sizeof(struct Parse_info_struct));
+	pi->N_words = nwords;
 	pi->parse_set = NULL;
 
-	if (pi->N_words >= 10) {
+	pi->chosen_disjuncts = (Disjunct **) xalloc(nwords * sizeof(Disjunct *));
+	memset(pi->chosen_disjuncts, 0, nwords * sizeof(Disjunct *));
+
+	pi->image_array = (Image_node **) xalloc(nwords * sizeof(Image_node *));
+	memset(pi->image_array, 0, nwords * sizeof(Image_node *));
+
+	pi->has_fat_down = (char *) xalloc(nwords * sizeof(Boolean));
+	memset(pi->has_fat_down, 0, nwords * sizeof(Boolean));
+
+	/* Alloc the x_table */
+	if (nwords >= 10) {
 		x_table_size = (1<<14);
-	} else if (pi->N_words >= 4) {
-		x_table_size = (1 << (pi->N_words));
+	} else if (nwords >= 4) {
+		x_table_size = (1 << nwords);
 	} else {
 		x_table_size = (1 << 4);
 	}
@@ -133,10 +144,9 @@ void init_x_table(Sentence sent)
 	/*printf("Allocating x_table of size %d\n", x_table_size);*/
 	pi->x_table_size = x_table_size;
 	pi->x_table = (X_table_connector**) xalloc(x_table_size * sizeof(X_table_connector*));
-	for (i=0; i<x_table_size; i++)
-	{
-		pi->x_table[i] = NULL;
-	}
+	memset(pi->x_table, 0, x_table_size * sizeof(X_table_connector*));
+
+	return pi;
 }
 
 /**
@@ -144,16 +154,15 @@ void init_x_table(Sentence sent)
  * it's a dag, a recursive free function won't work.  Every time we create
  * a set element, we put it in the hash table, so this is OK.
  */
-static void free_x_table(Parse_info pi)
+void free_parse_info(Parse_info pi)
 {
-	int i;
+	int i, len;
 	X_table_connector *t, *x;
 
-	if (pi->x_table == NULL)
-	{
-		/*fprintf(stderr, "Warning: Tried to free a NULL x_table\n");*/
-		return;
-	}
+	len = pi->N_words;
+	xfree(pi->chosen_disjuncts, len * sizeof(Disjunct *));
+	xfree(pi->image_array, len * sizeof(Image_node*));
+	xfree(pi->has_fat_down, len * sizeof(Boolean));
 
 	for (i=0; i<pi->x_table_size; i++)
 	{
@@ -164,10 +173,14 @@ static void free_x_table(Parse_info pi)
 			xfree((void *) t, sizeof(X_table_connector));
 		}
 	}
+	pi->parse_set = NULL;
+
 	/*printf("Freeing x_table of size %d\n", x_table_size);*/
 	xfree((void *) pi->x_table, pi->x_table_size * sizeof(X_table_connector*));
 	pi->x_table_size = 0;
 	pi->x_table = NULL;
+
+	xfree((void *) pi, sizeof(struct Parse_info_struct));
 }
 
 /**
@@ -487,19 +500,6 @@ int build_parse_set(Sentence sent, int cost, Parse_Options opts)
 	return verify_set(sent->parse_info);
 }
 
-void free_parse_set(Sentence sent)
-{
-	if (NULL == sent->parse_info) return;
-
-	/* This uses the x_table to free the whole parse set (the set itself
-	 * cannot be used cause it's a dag).  Called from the outside world.
-	 */
-	free_x_table(sent->parse_info);
-	sent->parse_info->parse_set = NULL;
-	xfree((void *) sent->parse_info, sizeof(struct Parse_info_struct));
-	sent->parse_info = NULL;
-}
-
 static void initialize_links(Parse_info pi)
 {
 	int i;
