@@ -277,6 +277,12 @@ void free_sentence_expressions(Sentence sent)
 /**
  * Return true if the sentence contains a conjunction.  Assumes
  * is_conjunction[] has been initialized.
+ *
+ * This routine has been added to the public API as of version 4.7.0,
+ * as it is required to determine whether a fat linkage might restult
+ * during parsing. This is needed to optimize the optioanl use of fat
+ * links. The goal is to eventually deprecate fat linkages entirely;
+ * this routine is needed in the transition phase.
  */
 int sentence_contains_conjunction(Sentence sent)
 {
@@ -303,7 +309,7 @@ void free_sentence_disjuncts(Sentence sent)
 
 /**
  * Assumes that the sentence expression lists have been generated.
- * This does all the necessary pruning and building of and structures.
+ * This does all the necessary pruning and building of fat-link structures.
  */
 void prepare_to_parse(Sentence sent, Parse_Options opts)
 {
@@ -328,15 +334,22 @@ void prepare_to_parse(Sentence sent, Parse_Options opts)
 
 	sent->null_links = (opts->min_null_count > 0);
 
-	has_conjunction = sentence_contains_conjunction(sent);
+	if (opts->use_fat_links)
+	{
+		has_conjunction = sentence_contains_conjunction(sent);
+	}
+	else
+	{
+		has_conjunction = FALSE;
+	}
+
+	/* The deletable region depends on whether null links are in use;
+	 * with null_links everything is deletable. Thus, this processing
+	 * cannot be done earlier than here.
+	 */
 	set_connector_length_limits(sent, opts);
 	build_deletable(sent, has_conjunction);
 	build_effective_dist(sent, has_conjunction);
-	/* Why do we do these here instead of in
-	   first_prepare_to_parse() only?  The
-	   reason is that the deletable region
-	   depends on if null links are in use.
-	   With null_links everything is deletable */
 
 	if (!has_conjunction) {
 		pp_and_power_prune(sent, RUTHLESS, opts);
@@ -348,7 +361,7 @@ void prepare_to_parse(Sentence sent, Parse_Options opts)
 			print_disjunct_counts(sent);
 		}
 		*/
-		/*print_time(opts, "Finished gentle power pruning"); */
+		/* already reported -- print_time(opts, "Finished gentle power pruning"); */
 		conjunction_prune(sent, opts);
 		if (verbosity > 2) {
 			printf("\nAfter conjunction pruning:\n");
